@@ -7,8 +7,16 @@
 import numpy as np
 import pandas
 import datetime
-import gzip, bz2
-import os, re, struct, warnings
+import gzip
+import bz2
+import os
+import re
+import struct
+import warnings
+
+#### SUB-MODULE DEPENDENCIES
+import ict
+import nas
 
 
 
@@ -63,12 +71,20 @@ class Naspy(object):
         Fifo(self).filetype_warnings() # Issue gzip warning
         
         return None
-        
-    def make_numpy(self, masked=False, missing_values='auto'):
+    
+       
+    def make_numpy(self, var_names='columns', masked=False, missing_values='auto'):
         """Generates a numpy.ndarray.
         
         PARAMETERS
         ----------
+        var_names : list or {'columns' or 'header'}, default 'columns'
+            Specifies how to determine variable names used in the numpy array; default 
+            behavior is to extract column names from the last line in the header. This is 
+            specified in ICARTT files but not always in NAS files. If this fails column 
+            names will be determined from variable descriptions in the header; this can 
+            be also be done by specifying 'header'. Alternately, a list of strings 
+            specifying variable names can be passed. Note that the case_sensitive options             are applied to this list.
         masked : bool, default False
             Species whether to return a masked array; must be True to convert missing 
             values to nans in the returned array.
@@ -92,10 +108,17 @@ class Naspy(object):
         arr = naspy.make_numpy(masked=True)
         
         """
+        # set delimiter
+        data_format = Fifo(self).get_filetype()[2]
+        if data_format == 'ict':
+            delimiter = ','
+        elif data_format == 'nas':
+            delimiter = None  # consecutive whitespaces is default
+        else:
+            raise AttributeError("unknown data format!")
+        
         # Get variable names
-        names = [ self.header.INDEPENDENT_VARIABLE['NAME'] ]
-        for i in range(len(self.header.DEPENDENT_VARIABLE)):
-            names.append(self.header.DEPENDENT_VARIABLE[i]['NAME'])
+        names = self.get_column_names(var_names)
         
         # Get a dictionary of missingVals on a per column basis
         if missing_values == 'auto':
@@ -103,17 +126,18 @@ class Naspy(object):
         else:
             missingVals = self.missing_values(missing_values)
         
+        
         # Read the lines into numpy object
         arr = np.genfromtxt(
                     self._fileAbsPath_,
-                    delimiter = ',',  # Comma seperated (ICT/AMES style?),
+                    delimiter = delimiter,  # Comma seperated (ICT/AMES style?),
                     names = names,
                     skip_header = self.header.HEADER_LINES,
                     missing_values = missingVals,
                     usemask=masked,
                     dtype = None,
                     case_sensitive = False,
-                    unpack=True
+                    # unpack=True
                     )
         # TODO: Future feature, numpy array with datetime64 support
         # KNOWN DEFECT: can't append datetime64 to an existing array using:
@@ -124,13 +148,18 @@ class Naspy(object):
         
         return arr
     
-    def make_DataFrame(self, case_sensitive='upper', convert_missing=True, 
-                        make_datetime=True, datetime_asindex=True, drop_datetime=True,
-                        missing_values='auto'):
+    def make_DataFrame(self, var_names='columns', case_sensitive='upper', 
+                       convert_missing=True, make_datetime=True, datetime_asindex=True,                          drop_datetime=True, missing_values='auto'):
         """Generates a pandas.DataFrame from a Naspy object.
         
         PARAMETERS
         ----------
+        var_names : list or {'columns' or 'header'}, default 'columns'
+            Specifies how to determine variable names used in the DataFrame; default                  behavior is to extract column names from the last line in the header. This is 
+            specified in ICARTT files but not always in NAS files. If this fails column 
+            names will be determined from variable descriptions in the header; this can 
+            be also be done by specifying 'header'. Alternately, a list of strings 
+            specifying variable names can be passed. Note that the case_sensitive options             are applied to this list.
         case_sensitive : string or bool, default 'upper'
             Formatting option of columns names; choices are 'upper' (default), 'lower', or
             'as-is' in addition to True or False. With no format specified, column names 
@@ -175,11 +204,20 @@ class Naspy(object):
         if ext not in ['gzip', 'bz2']:
             ext = None
         
+        # set delimiter
+        data_format = Fifo(self).get_filetype()[2]
+        if data_format == 'ict':
+            delimiter = ','
+        elif data_format == 'nas':
+            delimiter = r'\s*'
+        else:
+            raise AttributeError("unknown data format!")
+        
         df = pandas.io.parsers.read_table(
                  self._fileAbsPath_, 
                  skiprows = self.header.HEADER_LINES, 
-                 delimiter = ',', 
-                 names = self.get_column_names(case_sensitive),
+                 delimiter = delimiter, 
+                 names = self.get_column_names(var_names, case_sensitive),
                  dtype = np.float64, # Allow for nans
                  compression = ext
                  )
@@ -196,11 +234,18 @@ class Naspy(object):
         
         return df
     
-    def get_column_names(self, case_sensitive='upper'):
+    def get_column_names(self, var_names='columns', case_sensitive='upper'):
         """Returns a list of column names from a Naspy object.
         
         PARAMETERS
         ----------
+        var_names : list or {'columns' or 'header'}, default 'columns'
+            Specifies how variable names are derived; either from variable descriptions 
+            in the header (`header`) or from the last line of the header (`columns`); the 
+            default behavior is to get variable names from the columns with a fallback to 
+            header descriptions. Alternately the variable names can be manually passed  
+            with a list of strings. Note that case_sensitive options are still applied to 
+            this list.
         case_sensitive : string or bool, default 'upper'
             Formatting option of columns names; choices are 'upper' (default), 'lower', or
             'as-is' in addition to True or False. With no format specified, column names 
@@ -208,9 +253,22 @@ class Naspy(object):
             as they are specified in the file.
             
         """
-        names = [ self.header.INDEPENDENT_VARIABLE['NAME'] ]
-        for i in range(len(self.header.DEPENDENT_VARIABLE)):
-            names.append(self.header.DEPENDENT_VARIABLE[i]['NAME'])
+        if var_names == 'columns':
+            try:
+                names = self.header.COLUMN_VARIABLES
+            except AttributeError:
+                names = self.get_header_vars()
+        elif var_names == 'header':
+                names = self.get_header_vars()
+        elif type(var_names) == list:
+            if len(var_names) == int(self.header.TOTAL_NUM_VARIABLES):
+                names = var_names
+            else:
+                raise NameError("Number of variables in var_names list (%s) is "
+                                "inconsistent with the total number of variables (%s)" % 
+                                (len(var_names), self.header.TOTAL_NUM_VARIABLES) )
+        else:
+            raise AttributeError("Unknown var_names option: '%s'" % var_names)
         
         if case_sensitive in ['upper', False]:
             names = [name.upper() for name in names]
@@ -223,6 +281,13 @@ class Naspy(object):
         
         return names
     
+    def get_header_vars(self):
+        """Gets variable names from the header description."""
+        names = [ self.header.INDEPENDENT_VARIABLE['NAME'] ]
+        for i in range(len(self.header.DEPENDENT_VARIABLE)):
+            names.append(self.header.DEPENDENT_VARIABLE[i]['NAME'])
+        return names
+    
     def dataframe_nans(self, df, missing_values='auto'):
         """missing_values is a dictionary.
         """
@@ -279,10 +344,25 @@ class Naspy(object):
         
         return missingVals
     
+    def print_header(self):
+        """Displays the contents of the header to the screen."""
+        f = Fifo(self).open_file()
+        lines = []
+        i = 0
+        while i < self.header.HEADER_LINES:
+            lines.append(f.readline())
+            i += 1
+        f.close()
+        print ''
+        for line in lines:
+            print line.rstrip('\n')
+        return None
+    
     def __repr__(self):
         return ("%s Data File (FFI = %i)\n%s" % 
                 (self._file.data_format.upper(),
-                 self.header.FFI,
+                 # self.header.FFI,
+                 1001,
                  self.header.FILENAME) )
 
 
@@ -303,82 +383,17 @@ class Header(object):
     # Entry point to header parser
     def parse_header(self, Naspy):
         if Naspy._file.data_format == 'ict':
-            self.parse_header_ict()
-            self._parse_normal_comments_()
+            ict.parse_header(self)
+            # self.parse_header_ict()
+            # self._parse_normal_comments_()
         if Naspy._file.data_format == 'nas':
-            raise AttributeError("NASA Ames file format is not currently supported")
+            nas.parse_header(self)
+            # raise AttributeError("NASA Ames file format is not currently supported")
         
         return None
         
     
-    # TODO: create similar to parse NAS header; there are a few subtle differences between the NAS and ICT, so a seperate nas function is best
-    def parse_header_ict(self):
-        """Parses an ICT header and returns a naspy.header object. """
-        # get the filetype and specify the seperator
-        
-        f = self._fileObj_
-        self.HEADER_LINES, self.FFI = map(int, f.readline().split(','))
-        self.PI = f.readline().strip()
-        self.ORGANIZATION = f.readline().strip()
-        self.DATA_DESCRIPTION = f.readline().strip()
-        self.MISSION = f.readline().strip()
-        self.FILE_VOL_NO, self.NO_VOL = map(int, f.readline().split(','))
-
-        i = map(int, f.readline().split(','))
-        self.START_UTC = datetime.datetime(i[0],i[1],i[2]) # UTC date when data begin
-        self.REV_UTC =  datetime.date(i[3],i[4],i[5]) # UTC date of data red or rev
-
-        self.DATA_INTERVAL = float(f.readline())
-        
-        # Generate a dictionary for INDEPENDENT_VARIABLE
-        j = f.readline().strip().split(',') # Read Indepent_Variable line
-        for k in range(len(j),3):  # Ensure all fields are filled
-            try:
-                j[k] = None
-            except IndexError:
-                j.append(None)
-        self.INDEPENDENT_VARIABLE = {'NAME':j[0], 'UNITS':j[1], 'DESC':j[2]}  
-        
-        self.TOTAL_NUM_VARIABLES = int(f.readline().strip())+1
-        self.SCALE_FACTORS = self.SCALE_FACTORS = map(float, f.readline().split(','))
-        # self.MISSING_DATA_FLAGS = map(float, f.readline().split(','))
-        self.MISSING_DATA_FLAGS = f.readline().strip().replace(" ","").split(',')
-
-        # Create a dictionary for dependent variables
-        DEP_VAR = []
-        for i in range(1,self.TOTAL_NUM_VARIABLES):
-            j = f.readline().strip().split(',')
-            for k in range(len(j),3):  # Ensure all fields are filled
-                try:
-                    j[k] = None
-                except IndexError:
-                    j.append(None)
-            DEP_VAR.append({'NAME':j[0], 'UNITS':j[1], 'DESC':j[2]})
-        self.DEPENDENT_VARIABLE = DEP_VAR
-
-        self.SPECIAL_COMMENT_LINES = int(f.readline().strip())
-
-        SPECIAL_COMMENTS = []
-        for i in range(self.SPECIAL_COMMENT_LINES):
-            SPECIAL_COMMENTS.append(f.readline().strip())
-        self.SPECIAL_COMMENTS = SPECIAL_COMMENTS
-
-        self.NORMAL_COMMENT_LINES = int(f.readline().strip())
-
-        NORMAL_COMMENTS = []
-        for i in range(self.NORMAL_COMMENT_LINES):
-            NORMAL_COMMENTS.append(f.readline().strip())
-        self.NORMAL_COMMENTS = NORMAL_COMMENTS
-
-        return None
-
-
-    def _parse_normal_comments_(self):
-        for i in self.NORMAL_COMMENTS:
-            comment = i.split(':',1)
-            if len(comment) == 2:
-                setattr(self, comment[0].upper().strip(), comment[1].strip())
-        return None
+    
     
     ### ADDITIONAL HEADER FUNCTIONS
     # TODO: build up header funtions 
@@ -398,10 +413,14 @@ class Header(object):
         %s
         """ % (self.PI, self.ORGANIZATION, self.PI_CONTACT_INFO)
         return None
-
+    
     def data_description(self):
         pass
             
+    # def __repr__(self):
+    #     """Defines how header objects are displayed."""
+    #     return "%s\n" % self.FILENAME
+    
 
 
 class Time(object):  # I don't know how to inherit properly here!
